@@ -26,8 +26,8 @@ import (
 
 	"github.com/stretchr/testify/require"
 
-	"github.com/pulumi/pulumi/pkg/v3/codegen/internal/test"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/pulumi/pulumi/pkg/v3/codegen/testing/test"
 	"github.com/pulumi/pulumi/sdk/v3/python"
 )
 
@@ -75,6 +75,7 @@ func TestGeneratePackage(t *testing.T) {
 			"python/py_compile": pyCompileCheck,
 			"python/test":       pyTestCheck,
 		},
+		TestCases: test.PulumiPulumiSDKTests,
 	})
 }
 
