@@ -15,8 +15,10 @@
 package codegen
 
 import (
-	"github.com/stretchr/testify/assert"
 	"testing"
+
+	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/stretchr/testify/assert"
 )
 
 func TestStringSetContains(t *testing.T) {
@@ -42,3 +44,20 @@ func TestStringSetSubtract(t *testing.T) {
 	assert.Equal(t, setEmpty, set1234.Subtract(set1234))
 	assert.Equal(t, set1234, set1234.Subtract(setEmpty))
 }
+
+func TestSimplifyInputUnion(t *testing.T) {
+	u1 := &schema.UnionType{
+		ElementTypes: []schema.Type{
+			&schema.InputType{ElementType: schema.StringType},
+			schema.NumberType,
+		},
+	}
+
+	u2 := SimplifyInputUnion(u1)
+	assert.Equal(t, &schema.UnionType{
+		ElementTypes: []schema.Type{
+			schema.StringType,
+			schema.NumberType,
+		},
+	}, u2)
+}
