@@ -39,6 +39,7 @@ type Import struct {
 	Version           *semver.Version // The provider version to use for the resource, if any.
 	PluginDownloadURL string          // The provider PluginDownloadURL to use for the resource, if any.
 	Protect           bool            // Whether to mark the resource as protected after import
+	Properties        []string        // Which properties to include (Defaults to required properties)
 }
 
 // ImportOptions controls the import process.
@@ -98,6 +99,7 @@ func NewImportDeployment(ctx *plugin.Context, target *Target, projectName tokens
 		source:       NewErrorSource(projectName),
 		preview:      preview,
 		providers:    reg,
+		newPlans:     newResourcePlan(target.Config),
 	}, nil
 }
 
@@ -166,7 +168,7 @@ func (i *importer) getOrCreateStackResource(ctx context.Context) (resource.URN,
 	typ, name := resource.RootStackType, fmt.Sprintf("%s-%s", projectName, stackName)
 	urn := resource.NewURN(stackName, projectName, "", typ, tokens.QName(name))
 	state := resource.NewState(typ, urn, false, false, "", resource.PropertyMap{}, nil, "", false, false, nil, nil, "",
-		nil, false, nil, nil, nil, "", 0)
+		nil, false, nil, nil, nil, "", 0, false)
 	// TODO(seqnum) should stacks be created with 1? When do they ever get recreated/replaced?
 	if !i.executeSerial(ctx, NewCreateStep(i.deployment, noopEvent(0), state)) {
 		return "", false, false
@@ -252,7 +254,7 @@ func (i *importer) registerProviders(ctx context.Context) (map[resource.URN]stri
 		}
 
 		state := resource.NewState(typ, urn, true, false, "", inputs, nil, "", false, false, nil, nil, "", nil, false,
-			nil, nil, nil, "", 0)
+			nil, nil, nil, "", 0, false)
 		// TODO(seqnum) should default providers be created with 1? When do they ever get recreated/replaced?
 		if issueCheckErrors(i.deployment, state, urn, failures) {
 			return nil, nil, false
@@ -339,7 +341,7 @@ func (i *importer) importResources(ctx context.Context) result.Result {
 
 		// Create the new desired state. Note that the resource is protected.
 		new := resource.NewState(urn.Type(), urn, true, false, imp.ID, resource.PropertyMap{}, nil, parent, imp.Protect,
-			false, nil, nil, provider, nil, false, nil, nil, nil, "", 1)
+			false, nil, nil, provider, nil, false, nil, nil, nil, "", 1, false)
 		steps = append(steps, newImportDeploymentStep(i.deployment, new))
 	}
 
