@@ -335,27 +335,33 @@ func (s *DeleteStep) Res() *resource.State    { return s.old }
 func (s *DeleteStep) Logical() bool           { return !s.replacing }
 
 func (s *DeleteStep) Apply(preview bool) (resource.Status, StepCompleteFunc, error) {
-	// Refuse to delete protected resources.
-	if s.old.Protect {
+	// Refuse to delete protected resources (unless we're replacing them in
+	// which case we will of checked protect elsewhere)
+	if !s.replacing && s.old.Protect {
 		return resource.StatusOK, nil, fmt.Errorf("unable to delete resource %q\n"+
 			"as it is currently marked for protection. To unprotect the resource, "+
-			"either remove the `protect` flag from the resource in your Pulumi"+
+			"either remove the `protect` flag from the resource in your Pulumi "+
 			"program and run `pulumi up` or use the command:\n"+
 			"`pulumi state unprotect '%s'`", s.old.URN, s.old.URN)
 	}
 
-	// Deleting an External resource is a no-op, since Pulumi does not own the lifecycle.
-	if !preview && !s.old.External {
-		if s.old.Custom {
-			// Invoke the Delete RPC function for this provider:
-			prov, err := getProvider(s)
-			if err != nil {
-				return resource.StatusOK, nil, err
-			}
+	if preview {
+		// Do nothing in preview
+	} else if s.old.External {
+		// Deleting an External resource is a no-op, since Pulumi does not own the lifecycle.
+	} else if s.old.RetainOnDelete {
+		// Deleting a "drop on delete" is a no-op as the user has explicitly asked us to not delete the resource.
+	} else if s.old.Custom {
+		// Not preview and not external and not Drop and is custom, do the actual delete
 
-			if rst, err := prov.Delete(s.URN(), s.old.ID, s.old.Outputs, s.old.CustomTimeouts.Delete); err != nil {
-				return rst, nil, err
-			}
+		// Invoke the Delete RPC function for this provider:
+		prov, err := getProvider(s)
+		if err != nil {
+			return resource.StatusOK, nil, err
+		}
+
+		if rst, err := prov.Delete(s.URN(), s.old.ID, s.old.Outputs, s.old.CustomTimeouts.Delete); err != nil {
+			return rst, nil, err
 		}
 	}
 
@@ -773,7 +779,7 @@ func (s *RefreshStep) Apply(preview bool) (resource.Status, StepCompleteFunc, er
 		s.new = resource.NewState(s.old.Type, s.old.URN, s.old.Custom, s.old.Delete, resourceID, inputs, outputs,
 			s.old.Parent, s.old.Protect, s.old.External, s.old.Dependencies, initErrors, s.old.Provider,
 			s.old.PropertyDependencies, s.old.PendingReplacement, s.old.AdditionalSecretOutputs, s.old.Aliases,
-			&s.old.CustomTimeouts, s.old.ImportID, s.old.SequenceNumber)
+			&s.old.CustomTimeouts, s.old.ImportID, s.old.SequenceNumber, s.old.RetainOnDelete)
 	} else {
 		s.new = nil
 	}
@@ -915,7 +921,8 @@ func (s *ImportStep) Apply(preview bool) (resource.Status, StepCompleteFunc, err
 	// differences between the old and new states are between the inputs and outputs.
 	s.old = resource.NewState(s.new.Type, s.new.URN, s.new.Custom, false, s.new.ID, read.Inputs, read.Outputs,
 		s.new.Parent, s.new.Protect, false, s.new.Dependencies, s.new.InitErrors, s.new.Provider,
-		s.new.PropertyDependencies, false, nil, nil, &s.new.CustomTimeouts, s.new.ImportID, s.new.SequenceNumber)
+		s.new.PropertyDependencies, false, nil, nil, &s.new.CustomTimeouts, s.new.ImportID,
+		s.new.SequenceNumber, s.new.RetainOnDelete)
 
 	// If this step came from an import deployment, we need to fetch any required inputs from the state.
 	if s.planned {
@@ -930,10 +937,33 @@ func (s *ImportStep) Apply(preview bool) (resource.Status, StepCompleteFunc, err
 		if !ok {
 			return resource.StatusOK, nil, fmt.Errorf("unknown resource type '%v'", s.new.Type)
 		}
-		for _, p := range r.InputProperties {
-			if p.IsRequired() {
-				k := resource.PropertyKey(p.Name)
-				s.new.Inputs[k] = s.old.Inputs[k]
+
+		// Get the import object and see if it had properties set
+		var inputProperties []string
+		for _, imp := range s.deployment.imports {
+			if imp.ID == s.old.ID {
+				inputProperties = imp.Properties
+				break
+			}
+		}
+
+		if len(inputProperties) == 0 {
+			logging.V(9).Infof("Importing %v with required properties", s.URN())
+
+			for _, p := range r.InputProperties {
+				if p.IsRequired() {
+					k := resource.PropertyKey(p.Name)
+					s.new.Inputs[k] = s.old.Inputs[k]
+				}
+			}
+		} else {
+			logging.V(9).Infof("Importing %v with supplied properties: %v", s.URN(), inputProperties)
+
+			for _, p := range inputProperties {
+				k := resource.PropertyKey(p)
+				if value, has := s.old.Inputs[k]; has {
+					s.new.Inputs[k] = value
+				}
 			}
 		}
 	} else {
@@ -1166,6 +1196,28 @@ func (op StepOp) Suffix() string {
 	return ""
 }
 
+// ConstrainedTo returns true if this operation is no more impactful than the constraint.
+func (op StepOp) ConstrainedTo(constraint StepOp) bool {
+	var allowed []StepOp
+	switch constraint {
+	case OpSame, OpDelete, OpRead, OpReadReplacement, OpRefresh, OpReadDiscard, OpDiscardReplaced,
+		OpRemovePendingReplace, OpImport, OpImportReplacement:
+		allowed = []StepOp{constraint}
+	case OpCreate:
+		allowed = []StepOp{OpSame, OpCreate}
+	case OpUpdate:
+		allowed = []StepOp{OpSame, OpUpdate}
+	case OpReplace, OpCreateReplacement, OpDeleteReplaced:
+		allowed = []StepOp{OpSame, OpUpdate, constraint}
+	}
+	for _, candidate := range allowed {
+		if candidate == op {
+			return true
+		}
+	}
+	return false
+}
+
 // getProvider fetches the provider for the given step.
 func getProvider(s Step) (plugin.Provider, error) {
 	if providers.IsProviderType(s.Type()) {
