@@ -40,3 +40,13 @@ func TestLoadV1Checkpoint(t *testing.T) {
 	assert.NotNil(t, chk.Latest)
 	assert.Len(t, chk.Latest.Resources, 30)
 }
+
+func TestLoadV3Checkpoint(t *testing.T) {
+	bytes, err := ioutil.ReadFile("testdata/checkpoint-v3.json")
+	assert.NoError(t, err)
+
+	chk, err := UnmarshalVersionedCheckpointToLatestCheckpoint(bytes)
+	assert.NoError(t, err)
+	assert.NotNil(t, chk.Latest)
+	assert.Len(t, chk.Latest.Resources, 30)
+}
