@@ -1,6 +1,7 @@
-﻿// Copyright 2016-2019, Pulumi Corporation
+﻿// Copyright 2016-2021, Pulumi Corporation
 
 using System;
+using System.Collections.Generic;
 
 namespace Pulumi
 {
@@ -22,5 +23,12 @@ namespace Pulumi
             : base(message, innerException)
         {
         }
+
+        public static RunException OutputsHaveIncorrectType(
+            IEnumerable<string> outputAttributeNames)
+        {
+            var message = $"Output(s) '{string.Join(", ", outputAttributeNames)}' have incorrect type. [Output] attributed properties must be instances of Output<T>.";
+            return new RunException(message);
+        }
     }
 }
