@@ -270,11 +270,24 @@ import collections.abc
 import functools
 import sys
 import typing
-from typing import Any, Callable, Dict, Iterator, Mapping, Optional, Tuple, Type, TypeVar, Union, cast, get_type_hints
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterator,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+    get_type_hints,
+)
 
 from . import _utils
 
-T = TypeVar('T')
+T = TypeVar("T")
 
 
 _PULUMI_NAME = "_pulumi_name"
@@ -287,22 +300,27 @@ _TRANSLATE_PROPERTY = "_translate_property"
 def is_input_type(cls: type) -> bool:
     return hasattr(cls, _PULUMI_INPUT_TYPE)
 
+
 def is_output_type(cls: type) -> bool:
     return hasattr(cls, _PULUMI_OUTPUT_TYPE)
 
 
 class _MISSING_TYPE:
     pass
+
+
 MISSING = _MISSING_TYPE()
 """
 MISSING is a singleton sentinel object to detect if a parameter is supplied or not.
 """
 
+
 class _Property:
     """
     Represents a Pulumi property. It is not meant to be created outside this module,
     rather, the property() function should be used.
     """
+
     def __init__(self, name: str, default: Any = MISSING) -> None:
         if not name:
             raise TypeError("Missing name argument")
@@ -332,21 +350,26 @@ def _properties_from_annotations(cls: type) -> Dict[str, _Property]:
 
     # Get annotations that are defined on this class (not base classes).
     # These are returned in the order declared on Python 3.6+.
-    cls_annotations = cls.__dict__.get('__annotations__', {})
+    cls_annotations = cls.__dict__.get("__annotations__", {})
 
     def get_property(cls: type, a_name: str, a_type: Any) -> _Property:
         default = getattr(cls, a_name, MISSING)
-        p = default if isinstance(default, _Property) else _Property(name=a_name, default=default)
+        p = (
+            default
+            if isinstance(default, _Property)
+            else _Property(name=a_name, default=default)
+        )
         p.type = a_type
         return p
 
     return {
-        name: get_property(cls, name, type)
-        for name, type in cls_annotations.items()
+        name: get_property(cls, name, type) for name, type in cls_annotations.items()
     }
 
 
-def _process_class(cls: type, signifier_attr: str, is_input: bool = False, setter: bool = False):
+def _process_class(
+    cls: type, signifier_attr: str, is_input: bool = False, setter: bool = False
+):
     # Get properties.
     props = _properties_from_annotations(cls)
 
@@ -371,18 +394,29 @@ def _process_class(cls: type, signifier_attr: str, is_input: bool = False, sette
             globals = sys.modules[cls.__module__].__dict__
         else:
             globals = {}
-        init_fn = _init_fn(props, globals, issubclass(cls, dict), not is_input and hasattr(cls, _TRANSLATE_PROPERTY))
+        init_fn = _init_fn(
+            props,
+            globals,
+            issubclass(cls, dict),
+            not is_input and hasattr(cls, _TRANSLATE_PROPERTY),
+        )
         setattr(cls, "__init__", init_fn)
 
     # Add an __eq__() method if the class doesn't have one.
     # There's no need for a __ne__ method, since Python will call __eq__ and negate it.
     if "__eq__" not in cls.__dict__:
         if issubclass(cls, dict):
+
             def eq_fn(self, other):
-                return type(other) is type(self) and getattr(dict, "__eq__")(other, self)
+                return type(other) is type(self) and getattr(dict, "__eq__")(
+                    other, self
+                )
+
         else:
+
             def eq_fn(self, other):
                 return type(other) is type(self) and other.__dict__ == self.__dict__
+
         setattr(cls, "__eq__", eq_fn)
 
 
@@ -390,15 +424,19 @@ def _create_py_property(a_name: str, pulumi_name: str, typ: Any, setter: bool =
     """
     Returns a Python property getter that looks up the value using get.
     """
+
     def getter_fn(self):
         return get(self, a_name)
+
     getter_fn.__name__ = a_name
     getter_fn.__annotations__ = {"return": typ}
     setattr(getter_fn, _PULUMI_NAME, pulumi_name)
 
     if setter:
+
         def setter_fn(self, value):
             return set(self, a_name, value)
+
         setter_fn.__name__ = a_name
         setter_fn.__annotations__ = {"value": typ}
         return builtins.property(fget=getter_fn, fset=setter_fn)
@@ -422,7 +460,9 @@ def input_type(cls: Type[T]) -> Type[T]:
     """
 
     if is_input_type(cls) or is_output_type(cls):
-        raise AssertionError("Cannot apply @input_type and @output_type more than once.")
+        raise AssertionError(
+            "Cannot apply @input_type and @output_type more than once."
+        )
 
     # Get the input properties and mark the class as an input type.
     _process_class(cls, _PULUMI_INPUT_TYPE, is_input=True, setter=True)
@@ -431,6 +471,7 @@ def input_type(cls: Type[T]) -> Type[T]:
     def create_setter(name: str) -> Callable:
         def setter_fn(self, value):
             set(self, name, value)
+
         return setter_fn
 
     # Now, process the class's properties, replacing properties with empty setters with
@@ -451,7 +492,10 @@ def input_type_py_to_pulumi_names(input_type_cls: Type) -> Dict[str, str]:
     Returns a dict of Python names to Pulumi names for the input type.
     """
     assert is_input_type(input_type_cls)
-    return {python_name: pulumi_name for python_name, pulumi_name, _ in _py_properties(input_type_cls)}
+    return {
+        python_name: pulumi_name
+        for python_name, pulumi_name, _ in _py_properties(input_type_cls)
+    }
 
 
 def input_type_types(input_type_cls: type) -> Dict[str, type]:
@@ -503,7 +547,9 @@ def output_type(cls: Type[T]) -> Type[T]:
     """
 
     if is_input_type(cls) or is_output_type(cls):
-        raise AssertionError("Cannot apply @input_type and @output_type more than once.")
+        raise AssertionError(
+            "Cannot apply @input_type and @output_type more than once."
+        )
 
     # Get the output properties and mark the class as an output type.
     _process_class(cls, _PULUMI_OUTPUT_TYPE)
@@ -541,6 +587,7 @@ def getter(_fn=None, *, name: Optional[str] = None):
 
     name is the Pulumi property name. If not set, the name of the function is used.
     """
+
     def decorator(fn: Callable) -> Callable:
         if not callable(fn):
             raise TypeError("Expected fn to be callable")
@@ -548,10 +595,12 @@ def getter(_fn=None, *, name: Optional[str] = None):
         # If name isn't specified, use the name of the function.
         pulumi_name = name if name is not None else fn.__name__
         if _utils.is_empty_function(fn):
+
             @functools.wraps(fn)
             def get_fn(self):
                 # Get the value using the Python name, which is the name of the function.
                 return get(self, fn.__name__)
+
             fn = get_fn
         setattr(fn, _PULUMI_NAME, pulumi_name)
         return fn
@@ -604,10 +653,13 @@ def get(self, name: str) -> Any:
 
     # pylint: disable=import-outside-toplevel
     from . import Resource
+
     if isinstance(self, Resource):
         return self.__dict__.get(name)
 
-    raise AssertionError("get can only be used with classes decorated with @input_type or @output_type")
+    raise AssertionError(
+        "get can only be used with classes decorated with @input_type or @output_type"
+    )
 
 
 def set(self, name: str, value: Any) -> None:
@@ -631,7 +683,9 @@ def set(self, name: str, value: Any) -> None:
             self.__dict__[name] = value
         return
 
-    raise AssertionError("set can only be used with classes decorated with @input_type or @output_type")
+    raise AssertionError(
+        "set can only be used with classes decorated with @input_type or @output_type"
+    )
 
 
 # Use the built-in `get_origin` and `get_args` functions on Python 3.8+,
@@ -642,6 +696,7 @@ if sys.version_info[:2] >= (3, 8):
     # pylint: disable=no-member
     get_args = typing.get_args  # type: ignore
 elif sys.version_info[:2] >= (3, 7):
+
     def get_origin(tp):
         if isinstance(tp, typing._GenericAlias):  # type: ignore
             return tp.__origin__
@@ -651,7 +706,9 @@ elif sys.version_info[:2] >= (3, 7):
         if isinstance(tp, typing._GenericAlias):  # type: ignore
             return tp.__args__
         return ()
+
 else:
+
     def get_origin(tp):
         if hasattr(tp, "__origin__"):
             return tp.__origin__
@@ -662,8 +719,13 @@ else:
         if _is_union_type(tp) and hasattr(tp, "_subs_tree"):
             tree = tp._subs_tree()
             if isinstance(tree, tuple) and len(tree) > 1:
+
                 def _eval(args):
-                    return tuple(arg if not isinstance(arg, tuple) else arg[0][_eval(arg[1:])] for arg in args)
+                    return tuple(
+                        arg if not isinstance(arg, tuple) else arg[0][_eval(arg[1:])]
+                        for arg in args
+                    )
+
                 return _eval(tree[1:])
         if hasattr(tp, "__args__"):
             return tp.__args__
@@ -672,8 +734,11 @@ else:
 
 def _is_union_type(tp):
     if sys.version_info[:2] >= (3, 7):
-        return (tp is Union or
-                isinstance(tp, typing._GenericAlias) and tp.__origin__ is Union)  # type: ignore
+        return (
+            tp is Union
+            or isinstance(tp, typing._GenericAlias)
+            and tp.__origin__ is Union
+        )  # type: ignore
     # pylint: disable=unidiomatic-typecheck, no-member
     return type(tp) is typing._Union  # type: ignore
 
@@ -751,15 +816,13 @@ def _types_from_py_properties(cls: type) -> Dict[str, type]:
 
 def _pulumi_to_py_names_from_py_properties(cls: type) -> Dict[str, str]:
     return {
-        pulumi_name: python_name
-        for python_name, pulumi_name, _ in _py_properties(cls)
+        pulumi_name: python_name for python_name, pulumi_name, _ in _py_properties(cls)
     }
 
 
 def _py_to_pulumi_names_from_py_properties(cls: type) -> Dict[str, str]:
     return {
-        python_name: pulumi_name
-        for python_name, pulumi_name, _ in _py_properties(cls)
+        python_name: pulumi_name for python_name, pulumi_name, _ in _py_properties(cls)
     }
 
 
@@ -813,7 +876,7 @@ def _types_from_annotations(cls: type) -> Dict[str, type]:
 def _names_from_annotations(cls: type) -> Iterator[Tuple[str, str]]:
     # Get annotations that are defined on this class (not base classes).
     # These are returned in the order declared on Python 3.6+.
-    cls_annotations = cls.__dict__.get('__annotations__', {})
+    cls_annotations = cls.__dict__.get("__annotations__", {})
 
     def get_pulumi_name(a_name: str) -> str:
         default = getattr(cls, a_name, MISSING)
@@ -917,32 +980,40 @@ def unwrap_type(val: type) -> type:
 
     # If it looks like Input[T] or InputType[T], extract the T arg.
     if _is_union_type(val):
+
         def isInputType(args):
             assert len(args) > 1
-            return (is_input_type(args[0]) and
-                args[1] is dict or get_origin(args[1]) in {dict, Dict, Mapping, collections.abc.Mapping})
+            return (
+                is_input_type(args[0])
+                and args[1] is dict
+                or get_origin(args[1]) in {dict, Dict, Mapping, collections.abc.Mapping}
+            )
 
-        def isInput(args, i = 1):
+        def isInput(args, i=1):
             assert len(args) > i + 1
-            return (get_origin(args[i]) in {typing.Awaitable, collections.abc.Awaitable} and
-                get_origin(args[i + 1]) is Output)
+            return (
+                get_origin(args[i]) in {typing.Awaitable, collections.abc.Awaitable}
+                and get_origin(args[i + 1]) is Output
+            )
 
         args = get_args(val)
         if len(args) == 2:
-            if isInputType(args): # InputType[T]
+            if isInputType(args):  # InputType[T]
                 return args[0]
         elif len(args) == 3:
-            if isInput(args): # Input[T]
+            if isInput(args):  # Input[T]
                 return args[0]
-            if isInputType(args) and args[2] is type(None): # Optiona[InputType[T]]
+            if isInputType(args) and args[2] is type(None):  # Optiona[InputType[T]]
                 return args[0]
         elif len(args) == 4:
-            if isInput(args) and args[3] is type(None): # Optional[Input[T]]
+            if isInput(args) and args[3] is type(None):  # Optional[Input[T]]
                 return args[0]
-            if isInputType(args) and isInput(args, 2): # Input[InputType[T]]
+            if isInputType(args) and isInput(args, 2):  # Input[InputType[T]]
                 return args[0]
         elif len(args) == 5:
-            if isInputType(args) and isInput(args, 2) and args[4] is type(None): # Optional[Input[InputType[T]]]
+            if (
+                isInputType(args) and isInput(args, 2) and args[4] is type(None)
+            ):  # Optional[Input[InputType[T]]]
                 return args[0]
 
     return unwrap_optional_type(val)
@@ -951,6 +1022,7 @@ def unwrap_type(val: type) -> type:
 # The following functions for creating an __init__() method were adapted
 # from Python's dataclasses module.
 
+
 def _create_fn(name, args, body, *, globals=None, locals=None):
     if locals is None:
         locals = {}
@@ -970,7 +1042,9 @@ def _create_fn(name, args, body, *, globals=None, locals=None):
     return ns["__create_fn__"](**locals)
 
 
-def _property_init(python_name: str, prop: _Property, globals, is_dict: bool, has_translate: bool):
+def _property_init(
+    python_name: str, prop: _Property, globals, is_dict: bool, has_translate: bool
+):
     # Return the text of the line in the body of __init__() that will
     # initialize this property.
 
@@ -1028,13 +1102,16 @@ def _init_fn(props: Dict[str, _Property], globals, is_dict: bool, has_translate:
         if prop.default is not MISSING:
             seen_default = True
         elif seen_default:
-            raise TypeError(f"non-default argument {python_name!r} "
-                            "follows default argument")
+            raise TypeError(
+                f"non-default argument {python_name!r} " "follows default argument"
+            )
 
     locals = {f"_type_{python_name}": prop.type for python_name, prop in props.items()}
-    locals.update({
-        "MISSING": MISSING,
-    })
+    locals.update(
+        {
+            "MISSING": MISSING,
+        }
+    )
 
     body_lines = []
     for python_name, prop in props.items():
@@ -1050,8 +1127,11 @@ def _init_fn(props: Dict[str, _Property], globals, is_dict: bool, has_translate:
     if len(props) > 0:
         first_args += ["*"]
 
-    return _create_fn("__init__",
-                      first_args + [_init_param(python_name, prop) for python_name, prop in props.items()],
-                      body_lines,
-                      locals=locals,
-                      globals=globals)
+    return _create_fn(
+        "__init__",
+        first_args
+        + [_init_param(python_name, prop) for python_name, prop in props.items()],
+        body_lines,
+        locals=locals,
+        globals=globals,
+    )
