@@ -19,7 +19,7 @@ from pulumi import ResourceOptions, Input, Inputs
 
 class ConstructResult:
     """ConstructResult represents the results of a call to
-`Provider.construct`.
+    `Provider.construct`.
 
     """
 
@@ -43,7 +43,9 @@ class CheckFailure:
     reason: str
     """The reason that the property failed validation."""
 
-    def __init__(self, property: str, reason: str) -> None:  # pylint: disable=redefined-builtin
+    def __init__(
+        self, property: str, reason: str  # pylint: disable=redefined-builtin
+    ) -> None:
         self.property = property
         self.reason = reason
 
@@ -57,7 +59,9 @@ class CallResult:
     failures: Optional[Sequence[CheckFailure]]
     """Any validation failures that occurred."""
 
-    def __init__(self, outputs: Inputs, failures: Optional[Sequence[CheckFailure]] = None) -> None:
+    def __init__(
+        self, outputs: Inputs, failures: Optional[Sequence[CheckFailure]] = None
+    ) -> None:
         self.outputs = outputs
         self.failures = failures
 
@@ -79,8 +83,13 @@ class Provider:
         self.version = version
         self.schema = schema
 
-    def construct(self, name: str, resource_type: str, inputs: Inputs,
-                  options: Optional[ResourceOptions] = None) -> ConstructResult:
+    def construct(
+        self,
+        name: str,
+        resource_type: str,
+        inputs: Inputs,
+        options: Optional[ResourceOptions] = None,
+    ) -> ConstructResult:
         """Construct creates a new component resource.
 
         :param str name: The name of the resource to create.
@@ -98,4 +107,4 @@ class Provider:
         :param Inputs args: The inputs to the function.
         """
 
-        raise Exception(f'Unknown method {token}')
+        raise Exception(f"Unknown method {token}")
